###################
###### install and run required R package ###### 
library("statmod")
library("lme4")
library("lmerTest")
library("hrbrthemes")
library("viridis")
library("Matrix")
library("rsq")
library(rsq)
library(lme4)
library(tidyverse) # data wrangling and visualization
library(sjPlot)    # to visualizing mixed-effects models
library(effects)   # to visualizing mixed-effects models
library(lmerTest)  # p-values for MEMs based on the Satterthwaite approximation
library(report)    # mainly for an "report" function
library(emmeans)   # post-hoc analysis
library(knitr)     # beautifying tables
library(sjstats)   # ICC - intraclass-correlation coefficient
library(caret)     # ML, model comparison & utility functions
library(readxl)
library(tidyverse)
library(hrbrthemes)
library(viridis)
library(ggplot2)
library(RColorBrewer)
library(xgboost)
library(reshape2)
library("statmod")
library(devtools)
library(treeshap)
library(cowplot)
rm(list=ls())



###################
##### XGBoost ####

devtools::install_github('ModelOriented/treeshap')

#change to appropriate working directory
setwd("C:/Users/BootMR/OneDrive - Universiteit Twente/Various docs/RQ2a - shared flow/dataset to be published/Processed data")

###### import dataset from external excel file ######
df <- read_excel("data as input for analysis.xlsx")


# check for NAs
which(is.na(df$sf_intensity))

# Define the label (target variable)
y <- df$sf_intensity

# Convert y to a numeric vector (if not already)
#y <- as.numeric(y)

x <- as.matrix(df[,-c(1,9)])
head(x)

# Train an XGBoost model for regression
library(xgboost)

modelXG <- xgboost(data = x, label = y, objective = "reg:squarederror", max.depth = 7, nthread = 8, nround = 150, verbose = 0)



###################
##### determine multiple error rates for multiple objective functions

library(caret)

# Set the list of objective functions to try
objective_functions <- c("reg:squarederror", "reg:squaredlogerror", "reg:linear", "reg:tweedie", "reg:absoluteerror")

# Initialize a data frame to store the results
results <- data.frame(Objective = character(0), MAE = numeric(0), MSE = numeric(0), RMSE = numeric(0), R2 = numeric(0))

# Import dataset from an external Excel file
setwd("C:/Users/BootMR/OneDrive - Universiteit Twente/Various docs/RQ2a - shared flow/Untitled Folder")
df <- read_excel("data voor statistische analyse RQ1-flow-intensity.xlsx")

# Define the label (target variable)
y <- df$sf_intensity

# Convert y to a numeric vector (if not already)
y <- as.numeric(y)

x <- as.matrix(df[,-c(1,10)])

# Loop through each objective function
for (objective in objective_functions) {
  cat("Objective Function:", objective, "\n")
  
  # Train an XGBoost model for regression with the current objective
  modelXG <- xgboost(data = x, label = y, objective = objective, max.depth = 7, nthread = 8, nround = 150, verbose = 0)
  
  # Make predictions using the XGBoost model
  predictions <- predict(modelXG, x)
  
  # Calculate Mean Absolute Error (MAE)
  mae <- mean(abs(predictions - y))
  
  # Calculate Mean Squared Error (MSE)
  mse <- mean((predictions - y)^2)
  
  # Calculate Root Mean Squared Error (RMSE)
  rmse <- sqrt(mse)
  
  # Calculate R-squared (R2)
  r2 <- 1 - (sum((y - predictions)^2) / sum((y - mean(y))^2))
  
  # Store the results in the data frame
  results <- rbind(results, data.frame(Objective = objective, MAE = mae, MSE = mse, RMSE = rmse, R2 = r2))
  
  cat("Mean Absolute Error (MAE):", mae, "\n")
  cat("Mean Squared Error (MSE):", mse, "\n")
  cat("Root Mean Squared Error (RMSE):", rmse, "\n")
  cat("R-squared (R2):", r2, "\n\n")
}

# Sort the results by MAE in ascending order
sorted_results <- results[order(results$MAE), ]

# Display the sorted results
cat("Objective Functions Sorted by MAE:\n")
print(sorted_results)



###################
#### plotting the visualisations

TreeShapVal = treeshap(xgboost.unify(modelXG, x), x, interactions = FALSE, verbose = TRUE)

plot_feature_importance(TreeShapVal, max_vars = 50)


## plot HR

plot_HRmeanp1p2 <- plot_feature_dependence(TreeShapVal, "HRmeanp1p2", title = "HRmeanp1p2")
plot_meanHRmaxp1p2 <- plot_feature_dependence(TreeShapVal, "meanHRmaxp1p2", title = "meanHRmaxp1p2")
plot_diffHRmeanp1p2 <- plot_feature_dependence(TreeShapVal, "diffHRmeanp1p2", title = "diffHRmeanp1p2")
plot_HRmaxp1p2 <- plot_feature_dependence(TreeShapVal, "HRmaxp1p2", title = "HRmaxp1p2")
plot_HRdiffp1p2 <- plot_feature_dependence(TreeShapVal, "HRdiffp1p2", title = "HRdiffp1p2")

## plot Cadence

plot_Cdzerosp1p2 <- plot_feature_dependence(TreeShapVal, "Cdzerosp1p2", title = "Cdzerosp1p2")
plot_Cddiffp1p2 <- plot_feature_dependence(TreeShapVal, "Cddiffp1p2", title = "Cddiffp1p2")
plot_Cdmeanp1p2 <- plot_feature_dependence(TreeShapVal, "Cdmeanp1p2", title = "Cdmeanp1p2")

## plot others

plot_position <- plot_feature_dependence(TreeShapVal, "position", title = "Position")
plot_autotelic <- plot_feature_dependence(TreeShapVal, "autotelic", title = "Autotelic")
plot_condition <- plot_feature_dependence(TreeShapVal, "condition", title = "Condition")
plot_direction <- plot_feature_dependence(TreeShapVal, "direction", title = "direction")
plot_time <- plot_feature_dependence(TreeShapVal, "time", title = "Time")

## grid plots

plot_grid(plot_HRmeanp1p2, plot_meanHRmaxp1p2, plot_diffHRmeanp1p2, plot_HRmaxp1p2, plot_HRdiffp1p2)

plot_grid(plot_Cdzerosp1p2, plot_Cddiffp1p2, plot_Cdmeanp1p2)

plot_grid(plot_position, plot_autotelic, plot_condition, plot_direction, plot_time)











###################
#### trying a generalized linear regression model

df$balancing <- df$fitness

df$balancing <- factor(df$balancing, levels = c("no", "positive", "negative"), ordered = FALSE)
df$balancing <- relevel(df$balancing, ref = "no")

my_model <- lmer(sharedflowperc ~ balancing + autotelic + (1| pair), data = df)
summary(my_model)
model_summary <- summary(my_model)

fixed_effects <- model_summary$coefficients[2:nrow(model_summary$coefficients), ]
z_values <- fixed_effects[, "t value"]
p_values <- 2 * pt(abs(z_values), df.residual(my_model), lower.tail = FALSE)

# Combine the coefficients, z-values and p-values into a table
coefficients <- as.data.frame(fixed_effects[, "Estimate"])
coefficients$z_value <- z_values
coefficients$p_value <- p_values
rownames(coefficients) <- rownames(fixed_effects)
names(coefficients) <- c("Estimate", "z_value", "p_value")
print(coefficients)

coef_summary <- coef(summary(my_model))
z_intercept <- coef_summary["(Intercept)", "t value"]/coef_summary["(Intercept)", "Std. Error"]
p_intercept <- 2 * pt(abs(z_intercept), df.residual(my_model), lower.tail = FALSE)

# model glm proxy RQ1
model.simple <- glm(formula = sharedflow_logistic ~ position ,
                    family = binomial(link = "logit"), 
                    data = df)
summary(model.simple)

# model glmer proxy RQ1
model.pair <- glmer(sharedflow_logistic ~ position + Cddiffp1p2 + Cdzerosp1p2 + autotelic + (1|pair) + (1|condition) + (1|fitness), data=df, family = binomial)
summary(model.pair)

confint(model.pair)

full_var <- attr(summary(model.pair)$varcor, "sc")^2

# confidence intervals

se <- sqrt(diag(vcov(model.pair)))

# table of estimates with 95% CI
(tab <- cbind(Est = fixef(model.pair), LL = fixef(model.pair) - 1.96 * se, UL = fixef(model.pair) + 1.96 *
                se))
# odds ratios
exp(tab)

# coefficients of the model

coef(model.pair)

# histogram shared flow

counts <- table(df$sharedflow_logistic)
barplot(counts, main="Shared flow distribution",
        xlab="Shared flow")

# boxplot

box_plot = ggplot(df, aes(x = condition, y = diffHRmeanp1p2)) +  geom_boxplot() + geom_jitter(shape = 15,
color = "steelblue",
position = position_jitter(width = 0.21)) +
theme_classic()

# plot boxplot

box_plot

#stacked boxplot

bargraph <- ggplot(df, aes(x = sharedflow_logistic) + 
                     geom_bar(aes(y = (..count..)/sum(..count..))))

print(bargraph + labs(
  title = "Distribution of shared flow values",
  y = "Count", x = "Shared flow"))


# mosaic plot

mosaicplot(position ~ condition,data=df)

# plot x,y

ggplot(df,aes(autotelic,sharedflow_logistic)) + geom_point()
